from numba import jit
from cmath import sqrt
from math import tanh, atanh, pi
from lines import cLineIntersection

def deg2rad(rA):  return rA/180*pi
def rad2degd(rA): return rA*180/pi

@jit
def cap(rN):
    if rN < 0:
        return 0
    return rN
@jit
def ZeroCheck(cN):
    if cN == complex(0,0):
        return complex(1,0)
    else:
        return cN

@jit
def c_tr(cN):
    return cN.real, cN.imag

@jit
def cNorm(cN):
    return ZeroCheck(cN)/abs(ZeroCheck(cN))

@jit
def cDot(cA, cB):
    return (cA * cB.conjugate()).real

@jit
def cDist(cA, cB):
    return abs(cA-cB)

@jit
def MobiusInt(cA,cB,cC,cD): # Bruh
    return Klein2Poincare(cLineIntersection(Poincare2Klein(cA),Poincare2Klein(cB),Poincare2Klein(cC),Poincare2Klein(cD)))

@jit
def MobiusAdd(cA, cB):
    return (cA + cB) / (1 + cA.conjugate() * cB)

@jit
def MobiusGyr(cA, cB):
    return (1 + cA * cB.conjugate()) / (1 + cA.conjugate() * cB)

@jit
def MobiusAddGyr(cA, cB):
    return MobiusAdd(cA, cB), MobiusGyr(cA, cB)

@jit
def MobiusScalar(cA, rT):
    m = abs(cA)
    return tanh(rT * atanh(m)) * cA / m

@jit
def MobiusCoadd(cA,cB):
    return MobiusAdd(cA, (MobiusGyr(cA,-cB) * cB))

@jit
def MobiusCosub(cA,cB):
    return MobiusAdd(cA, -(MobiusGyr(cA,cB) * cB))

@jit
def MobiusLine(cA,cB,rT):
    return MobiusAdd(
            cA,
            MobiusScalar(
                MobiusAdd(-cA,cB),
                rT))

@jit
def MobiusDist(cA,cB):
    return abs(MobiusAdd(-cB,cA))

@jit
def Poincare2Klein(cN):
    return 2*cN / (1 + cDot(cN,cN))

@jit
def Klein2Poincare(cN):
    return (1-sqrt(1-cDot(cN,cN)))*cN/cDot(cN,cN)

class GyroVector:
    def __init__(self, cPos, cRot):
        self.cPos = complex(cPos)
        self.cRot = complex(cRot)
        self.normalize()

    def __add__(gA, gB):
        #cAdd, cGyr = MobiusAddGyr(gA.cPos, gB.cPos / gA.cRot)
        cAdd, cGyr = MobiusAddGyr(gA.cPos, gB.cPos)
        return GyroVector(cAdd, gA.cRot * gB.cRot * cGyr)

    def __neg__(self):
        return GyroVector(-(self.cRot * self.cPos), 1/self.cRot)

    def __sub__(gA, gB):
        return gA + (-gB)

    def copy(self):
        return GyroVector(self.cPos,self.cRot)
    
    def normalize(self):
        if abs(self.cPos) > 1:
            self.cPos = cNorm(self.cPos)
        self.cRot = cNorm(self.cRot)
    
    def rotate(self,cRot):
        self.cRot *= cRot

    def transform(self, cA):
        #cAdd, cGyr = MobiusAddGyr(self.cPos, cA / ZeroCheck(self.cRot))
        cAdd, cGyr = MobiusAddGyr(self.cPos, cA / self.cRot)
        self.cPos = cAdd
        self.cRot = self.cRot * cGyr

    def transformed(self, cA):
        #cAdd, cGyr = MobiusAddGyr(self.cPos, cA / ZeroCheck(self.cRot))
        #return GyroVector(cAdd, self.cRot * cGyr)
        return gA.copy().transform(cA)
    
    def nrtransformed(self, cA):
        return MobiusAdd(self.cPos, cA / self.cRot)

    def dist(self,gA):
        return abs((-self+gA).cPos)
